@@ -21,6 +21,7 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 # DEALINGS IN THE SOFTWARE.
 
+import argparse
 import os
 import os.path
 import re
@@ -30,6 +31,8 @@ import sys
 sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..')) # For access to sexps.py, which is in parent dir
 from sexps import *
 
+runner = ":"
+outdir = "."
 def make_test_case(f_name, ret_type, body):
     """Create a simple optimization test case consisting of a single
     function with the given name, return type, and body.
@@ -295,15 +298,15 @@ def create_test_case(doc_string, input_sexp, expected_sexp, test_name,
         'do_lower_jumps({0:d}, {1:d}, {2:d}, {3:d}, {4:d})'.format(
             pull_out_jumps, lower_sub_return, lower_main_return,
             lower_continue, lower_break))
-    args = ['../../glsl_test', 'optpass', '--quiet', '--input-ir', optimization]
-    test_file = '{0}.opt_test'.format(test_name)
+    args = [runner, 'optpass', '--quiet', '--input-ir', optimization]
+    test_file = os.path.join(outdir, '{0}.opt_test'.format(test_name))
     with open(test_file, 'w') as f:
         f.write('#!/usr/bin/env bash\n#\n# This file was generated by create_test_cases.py.\n#\n')
         f.write(doc_string)
         f.write('{0} <<EOF\n'.format(bash_quote(*args)))
         f.write('{0}\nEOF\n'.format(input_str))
     os.chmod(test_file, 0774)
-    expected_file = '{0}.opt_test.expected'.format(test_name)
+    expected_file = os.path.join(outdir, '{0}.opt_test.expected'.format(test_name))
     with open(expected_file, 'w') as f:
         f.write('{0}\n'.format(expected_output))
 
@@ -625,7 +628,10 @@ def test_lower_return_non_void_at_end_of_loop():
             loop(assign_x('a', const_float(1)) +
                  lowered_return_simple(const_float(2)) +
                  break_()) +
-            if_not_return_flag(assign_x('b', const_float(3)) +
+            if_return_flag(assign_x('return_value', '(var_ref return_value)') +
+                           assign_x('return_flag', const_bool(1)) +
+                           assign_x('execute_flag', const_bool(0)),
+                           assign_x('b', const_float(3)) +
                                lowered_return(const_float(4))) +
             final_return()
             ))
@@ -636,6 +642,17 @@ def test_lower_return_non_void_at_end_of_loop():
                      lower_sub_return=True, lower_break=True)
 
 if __name__ == '__main__':
+    parser = argparse.ArgumentParser()
+    parser.add_argument('--runner',
+                        help='The glsl_test runner',
+                        required=True)
+    parser.add_argument('--outdir',
+                        help='Directory to put the generated files in',
+                        required=True)
+    args = parser.parse_args()
+    runner = args.runner
+    outdir = args.outdir
+
     test_lower_returns_main()
     test_lower_returns_sub()
     test_lower_returns_1()
